#include "std/errno.h"
#include "thread/Scheduler.h"

#define ID_PID 0
#define ID_PPID 1
#define ID_UID 2
#define ID_EUID 3
#define ID_GID 4
#define ID_EGID 5

void sys_getprocid(Context* context, int field)
{
    if (field == ID_PID)
    {
        context->rax = Scheduler::current_task()->id;
        return;
    }
    else if (field == ID_PPID)
    {
        context->rax = Scheduler::current_task()->ppid;
        return;
    }
    else if (field == ID_UID)
    {
        context->rax = Scheduler::current_task()->uid;
        return;
    }
    else if (field == ID_EUID)
    {
        context->rax = Scheduler::current_task()->euid;
        return;
    }
    else if (field == ID_GID)
    {
        context->rax = Scheduler::current_task()->gid;
        return;
    }
    else if (field == ID_EGID)
    {
        context->rax = Scheduler::current_task()->egid;
        return;
    }
    else
    {
        context->rax = -EINVAL;
        return;
    }
}

void sys_setuid(Context* context, int new_uid, int new_euid)
{
    Task* current_task = Scheduler::current_task();

    if (!current_task->is_superuser())
    {
        if (new_uid != current_task->uid && new_uid != current_task->euid)
        {
            context->rax = -EPERM;
            return;
        }
        if (new_euid != current_task->euid && new_euid != current_task->uid)
        {
            context->rax = -EPERM;
            return;
        }
    }

    current_task->uid = new_uid;
    current_task->euid = new_euid;

    context->rax = 0;
}

void sys_setgid(Context* context, int new_gid, int new_egid)
{
    Task* current_task = Scheduler::current_task();

    if (!current_task->is_superuser())
    {
        if (new_gid != current_task->gid && new_gid != current_task->egid)
        {
            context->rax = -EPERM;
            return;
        }
        if (new_egid != current_task->egid && new_egid != current_task->gid)
        {
            context->rax = -EPERM;
            return;
        }
    }

    current_task->gid = new_gid;
    current_task->egid = new_egid;

    context->rax = 0;
}